@@ -26,9 +26,9 @@ int main()
 {
   using namespace __gnu_test;
 
-  // Equivalent, but formatting difference in void argument.
-  //  verify_demangle("_Z1fM1AFivEPS0_", "f(int (A::*)(void), int (*)(void))");
-  verify_demangle("_Z1fM1AFivEPS0_", "f(int (A::*)(), int (*)())");
+ // cplus-dem FAIL
+  verify_demangle("_Z3fooA30_A_i", "foo(int [30][])");
+
 
   return 0;
 }
